@@ -19,25 +19,25 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: festival.c,v 1.48 2004/04/04 21:33:22 hanke Exp $
+ * $Id: festival.c,v 1.54 2004/07/22 12:59:44 hanke Exp $
  */
 
 #include "fdset.h"
+#include "fdsetconv.h"
 
-#include "module_utils.c"
+#include "module_utils.h"
 #include "module_utils_audio.c"
 
 #include "festival_client.c"
 
 #define MODULE_NAME     "festival"
-#define MODULE_VERSION  "0.1"
+#define MODULE_VERSION  "0.2"
 
-#define DEBUG_MODULE 1
 DECLARE_DEBUG();
 
 /* Thread and process control */
 static pthread_t festival_speak_thread;
-static pid_t festival_pid = 0;
+static pid_t festival_child_pid = 0;
 static sem_t *festival_semaphore;
 static int festival_speaking = 0;
 static int festival_pause_requested = 0;
@@ -47,10 +47,26 @@ static EMessageType festival_message_type;
 static int festival_position = 0;
 signed int festival_volume = 0;
 
-char *festival_coding;
+int festival_stop = 0;
+
+int festival_process_pid = 0;
 
 FT_Info *festival_info;
 
+enum{
+    FCT_SOCKET = 0,
+    FCT_LOCAL = 1,
+} FestivalComType;
+
+struct{
+    int pipe_in[2];
+    int pipe_out[2];
+    int pid;
+}module_p;
+  
+#define COM_SOCKET ((FestivalComType == FCT_SOCKET) ? 1 : 0)
+#define COM_LOCAL ((FestivalComType == FCT_LOCAL) ? 1 : 0)
+
 /* Internal functions prototypes */
 void* _festival_speak(void*);
 
@@ -71,6 +87,11 @@ void festival_set_punctuation_mode(EPunctMode punct);
 void festival_set_cap_let_recogn(ECapLetRecogn recogn);
 void festival_set_volume(signed int volume);
 
+int init_festival_standalone();
+int init_festival_socket();
+
+MOD_OPTION_1_INT(FestivalComunicationType);
+
 MOD_OPTION_1_INT(FestivalMaxChunkLength);
 MOD_OPTION_1_STR(FestivalDelimiters);
 MOD_OPTION_1_STR(FestivalServerHost);
@@ -86,6 +107,8 @@ MOD_OPTION_1_INT(FestivalCacheDistinguishVoices);
 MOD_OPTION_1_INT(FestivalCacheDistinguishRate);
 MOD_OPTION_1_INT(FestivalCacheDistinguishPitch);
 
+MOD_OPTION_1_INT(FestivalReopenSocket);
+
 typedef struct{
     size_t size;
     GHashTable *caches;
@@ -110,7 +133,7 @@ TCache FestivalCache;
 int cache_init();
 int cache_reset();
 int cache_insert(char* key, EMessageType msgtype, FT_Wave *value);
-FT_Wave* cache_lookup(char *key, EMessageType msgtype, int add_counter);
+FT_Wave* cache_lookup(const char *key, EMessageType msgtype, int add_counter);
 
 /* Public functions */
 
@@ -122,16 +145,13 @@ module_load(void)
 
     REGISTER_DEBUG();
 
-    MOD_OPTION_1_INT_REG(FestivalMaxChunkLength, 300);
-    MOD_OPTION_1_STR_REG(FestivalDelimiters, ".");
+    MOD_OPTION_1_INT_REG(FestivalComunicationType, 0);
 
     MOD_OPTION_1_STR_REG(FestivalServerHost, "localhost");
     MOD_OPTION_1_INT_REG(FestivalServerPort, 1314);
 
     MOD_OPTION_1_INT_REG(FestivalDebugSaveOutput, 0);
 
-    MOD_OPTION_1_STR_REG(FestivalStripPunctChars, "");
-
     MOD_OPTION_1_STR_REG(FestivalRecodeFallback, "?");
 
     MOD_OPTION_1_INT_REG(FestivalCacheOn, 1);
@@ -140,43 +160,86 @@ module_load(void)
     MOD_OPTION_1_INT_REG(FestivalCacheDistinguishRate, 0);
     MOD_OPTION_1_INT_REG(FestivalCacheDistinguishPitch, 0);
 
+    /* TODO: Switch this option to 1 when the bug with the 40ms delay
+     in Festival is fixed */
+    MOD_OPTION_1_INT_REG(FestivalReopenSocket, 0);
+
     return 0;
 }
 
 int
-module_init(void)
+module_init(char **status_info)
 {
     int ret;
 
+    GString *info;
+
+    info = g_string_new("");
+
     INIT_DEBUG();
 
     DBG("module_init()");
 
-    /* Init festival and register a new voice */
-    festival_info = festivalDefaultInfo();
-    festival_info->server_host = FestivalServerHost;
-    festival_info->server_port = FestivalServerPort;
+    /* Initialize appropriate communication mechanism */
+    FestivalComType = FestivalComunicationType;
+    if (COM_SOCKET){
+	g_string_append(info, "Communicating with Festival through a socket. ");
+	ret = init_festival_socket();
+	if (ret == -1){
+	    g_string_append(info, "Can't connect to Festival server. Check your configuration "
+			    "in etc/speechd-modules/festival.conf for the specified host and port "
+			    "and check if Festival is really running there, e.g. with telnet. "
+			    "Please see documentation for more info.");
+	    *status_info = info->str;
+	    g_string_free(info, 0);
+	    return -1;
+	}else if (ret == -2){
+	    g_string_append(info, "Connect to the Festival server was successful, "
+			    "but I got disconnected immediately. This is most likely "
+			    "because of authorization problems. Check the variable "
+                            "server_access_list in etc/festival.scm and consult documentation "
+			    "for more information");
+	    *status_info = info->str;
+	    g_string_free(info, 0);
+	    return -1;
+	}
 
-    festival_info = festivalOpen(festival_info);
-    if (festival_info == NULL) return -2;
+    }
+    if (COM_LOCAL){
+	g_string_append(info, "Communicating with Festival through a local child process.");
+	if(init_festival_standalone()){
+	    g_string_append(info, "Local connect to Festival failed for unknown reasons.");
+	    *status_info = info->str;
+	    g_string_free(info, 0);
+	    return -1;
+	    
+	}
+    }
 
-    DBG("FestivalServerHost = %s\n", FestivalServerHost);
-    DBG("FestivalServerPort = %d\n", FestivalServerPort);
-    
+    /* Initialize global variables */
     festival_message = (char**) xmalloc (sizeof (char*));    
+
+    /* Initialize festival_speak thread to handle communication
+     with festival in a separate thread (to be faster in communication
+     with Speech Dispatcher) */
     festival_semaphore = module_semaphore_init();
     if (festival_semaphore == NULL) return -1;
-
-    festival_coding = g_strdup("ISO-8859-1");
-
     DBG("Festival: creating new thread for festival_speak\n");
     festival_speaking = 0;
     ret = pthread_create(&festival_speak_thread, NULL, _festival_speak, NULL);
     if(ret != 0){
         DBG("Festival: thread failed\n");
-        return -1;
+	g_string_append(info, "The module couldn't initialize threads"
+			"This can be either an internal problem or an"
+                        "architecture problem. If you are sure your architecture"
+			"supports threads, please report a bug.");
+	*status_info = info->str;
+	g_string_free(info, 0);
+	return -1;
     }
-								
+
+    *status_info = info->str;
+    g_string_free(info, 0);
     return 0;
 }
 
@@ -184,7 +247,7 @@ int
 module_speak(char *data, size_t bytes, EMessageType msgtype)
 {
 
-    DBG("module_write()\n");
+    DBG("module_speak()\n");
 
     if (data == NULL) return -1;
 
@@ -193,6 +256,8 @@ module_speak(char *data, size_t bytes, EMessageType msgtype)
         return -1;
     }
 
+    festival_stop = 0;
+
     /* Check the parameters */
     if(module_write_data_ok(data) != 0) return -1;
 
@@ -202,16 +267,26 @@ module_speak(char *data, size_t bytes, EMessageType msgtype)
 
     /* If the connection crashed or language or voice
        change, we will need to set all the parameters again */
-    if (festival_connection_crashed){
-        CLEAN_OLD_SETTINGS_TABLE();
-        festival_connection_crashed = 0;
+    if (COM_SOCKET){
+	if (festival_connection_crashed){
+	    DBG("Recovering after a connection loss");
+	    CLEAN_OLD_SETTINGS_TABLE();
+	    festival_info = festivalOpen(festival_info);
+	    FestivalSetMultiMode(festival_info, "t");
+	    festival_connection_crashed = 0;
+	}
     }
+
+    /* If the voice was changed, re-set all the parameters */
     if ((msg_settings.voice != msg_settings_old.voice) 
         || strcmp(msg_settings.language, msg_settings_old.language)){
+	DBG("Cleaning old settings table");
         CLEAN_OLD_SETTINGS_TABLE();
     }
 
+
     /* Setting voice parameters */
+    DBG("Updating parameters");
     UPDATE_STRING_PARAMETER(language, festival_set_language);
     UPDATE_PARAMETER(voice, festival_set_voice);
     UPDATE_PARAMETER(rate, festival_set_rate);
@@ -220,39 +295,53 @@ module_speak(char *data, size_t bytes, EMessageType msgtype)
     UPDATE_PARAMETER(punctuation_mode, festival_set_punctuation_mode);
     UPDATE_PARAMETER(cap_let_recogn, festival_set_cap_let_recogn);
 
-    DBG("Requested data: |%s| (recoding to %s)\n", data, festival_coding);
+    DBG("Requested data: |%s| \n", data);
 
-    *festival_message = (char*) g_convert_with_fallback(data, bytes, festival_coding, "UTF-8",
-                                              FestivalRecodeFallback, NULL, NULL, NULL);
+    *festival_message = strdup(data);
     if (*festival_message == NULL){
-        DBG("Error: Recoding unsuccessful.");
+        DBG("Error: Copying data unsuccesful.");
         return -1;
     }
-    DBG("Requested data after recoding: |%s|\n", *festival_message);
-    if(msgtype == MSGTYPE_TEXT){
-        module_strip_punctuation_some(*festival_message, FestivalStripPunctChars);
-        DBG("Requested after stripping punct: |%s|\n", *festival_message);
-    }else{
-        DBG("Non-textual event: |%s|\n", *festival_message);
-    }
 
     /* Send semaphore signal to the speaking thread */
-    festival_speaking = 1;    
-    sem_post(festival_semaphore);    
-		
+    festival_speaking = 1;
+    sem_post(festival_semaphore);
+
     DBG("Festival: leaving write() normaly\n\r");
     return bytes;
 }
 
 int
-module_stop(void) 
+module_stop(void)
 {
     DBG("stop()\n");
 
-    if(festival_speaking && (festival_pid != 0)){
-        DBG("stopping process pid %d\n", festival_pid);
-        kill(festival_pid, SIGKILL);
+    if(festival_speaking){
+	/* if(COM_SOCKET) */
+	if (0){
+	    if (festival_info != 0)
+		if ((festival_info->server_fd != -1) && FestivalReopenSocket){
+		    /* TODO: Maybe use shutdown here? */
+		    close(festival_info->server_fd);
+		    festival_info->server_fd = -1;
+		    festival_connection_crashed = 1;
+		    DBG("festival socket closed by module_stop()");
+		}
+	}
+	if (COM_LOCAL){
+	    DBG("festival local stopped by sending SIGINT");
+	    /* TODO: Write this function for local communication */
+	    //	    festival_stop_local();
+	}
+
+	if (festival_child_pid != 0){
+	    DBG("festival stopped by killing child process pid %d\n", festival_child_pid);
+	    kill(festival_child_pid, SIGKILL);
+	    festival_child_pid = 0;
+	}
     }
+
+    return 0;
 }
 
 size_t
@@ -260,14 +349,13 @@ module_pause(void)
 {
     DBG("pause requested\n");
     if(festival_speaking){
-
-        DBG("Sending request to pause to child\n");
+        DBG("Sending request for pause to child\n");
         festival_pause_requested = 1;
         DBG("Waiting in pause for child to terminate\n");
-        while(festival_speaking) usleep(10);
-
-        DBG("paused at byte: %d", current_index_mark);
-        return current_index_mark;        
+        while(festival_speaking) usleep(10000);       
+        DBG("Paused at mark: %d", current_index_mark);
+	festival_pause_requested = 0;
+        return current_index_mark;
     }else{
         return -1;
     }
@@ -290,13 +378,14 @@ module_close(int status)
         usleep(5);
     }
 
-    DBG("festivalClose()");
-    festivalClose(festival_info);
+    // DBG("festivalClose()");
+    // festivalClose(festival_info);
     
     module_terminate_thread(festival_speak_thread);
    
     delete_FT_Info(festival_info);
 
+    /* TODO: Solve this */
     DBG("Removing junk files in tmp/");
     system("rm -f /tmp/est* 2> /dev/null");
 
@@ -318,10 +407,10 @@ _festival_speak(void* nothing)
 
     cache_init();
 
-    module_speak_thread_wfork(festival_semaphore, &festival_pid,
+    module_speak_thread_wfork(festival_semaphore, &festival_child_pid,
                               module_audio_output_child, _festival_parent, 
                               &festival_speaking, festival_message, &festival_message_type,
-                              (size_t) FestivalMaxChunkLength, FestivalDelimiters,
+                              -1, NULL,
                               &festival_position, &festival_pause_requested);
     festival_speaking = 0;
 
@@ -330,171 +419,174 @@ _festival_speak(void* nothing)
     pthread_exit(NULL);
 }	
 
+
+#define CLEAN_UP(code) \
+        { \
+          if(!wave_cached) delete_FT_Wave(fwave); \
+          module_parent_dp_close(dpipe); \
+          return(code); \
+        }
+
+int
+is_text(EMessageType msg_type)
+{
+    if (msg_type == MSGTYPE_TEXT || msg_type == MSGTYPE_SPELL) return 1;
+    else return 0;
+}
+
+
 size_t
 _festival_parent(TModuleDoublePipe dpipe, const char* message,
-              EMessageType msgtype, const size_t maxlen, const char* dividers,
-              int *pause_requested)
+		 EMessageType msgtype, const size_t maxlen, const char* dividers,
+		 int *pause_requested)
 {
-    int pos = 0;
     int ret;
     int bytes;
-    FT_Wave *fwave;
-    char buf[4096];
+    FT_Wave *fwave = NULL;
     int terminate = 0;
-    int first_run = 1;
-    int r = -999;
+    int r = -1;
     static int debug_count = 0;
-    int o_bytes = 0;
-    int pause = -1;
-    int cont = 0;
+    int l;
+    int h;
+    char *tptr;
+    int wave_cached = 0;
+
+    char *callback;
 
     DBG("Entering parent process, closing pipes");
 
+    bytes = strlen(message);
+
     module_parent_dp_init(dpipe);
+    
+    DBG("Synthesizing: |%s|", message);
+    if (bytes > 0){
+	if (is_text(msgtype)){	/* is it a raw text */
+	    FestivalSetMultiMode(festival_info, "t");
+	}else{			/* it is some kind of event */
+	    FestivalSetMultiMode(festival_info, "nil");
+	    DBG("Cache mechanisms...");
+	    fwave = cache_lookup(message, msgtype, 1);
+	    if (fwave != NULL){
+		wave_cached = 1;
+		if (fwave->num_samples != 0){
+		    if(FestivalDebugSaveOutput){
+			char filename_debug[256];
+			sprintf(filename_debug, "/tmp/debug-festival-%d.snd", debug_count++);
+			save_FT_Wave_snd(fwave, filename_debug);
+		    }
+		    ret = module_parent_send_samples(dpipe, fwave->samples,
+						     fwave->num_samples, fwave->sample_rate,
+							 festival_volume);
+		    if (ret == -1) CLEAN_UP(-1);
+		    
+		    DBG("parent (1): Sent %d bytes\n", ret);            
+		    ret = module_parent_dp_write(dpipe, "\r\nOK_SPEECHD_DATA_SENT\r\n", 24);
+		    if (ret == -1) CLEAN_UP(-1);
+		    
+		    ret = module_parent_wait_continue(dpipe);
+		    if (ret != 0) CLEAN_UP(-1);
+		    
+		    DBG("End of data in parent (1), closing pipes");
+		    
+		    CLEAN_UP(0);
+		}else
+		    CLEAN_UP(0);
+	    }
+	}
+	
+	switch(msgtype)
+	    {
+	    case MSGTYPE_TEXT: r = festivalStringToWaveRequest(festival_info, message); break;
+	    case MSGTYPE_SOUND_ICON: r = festivalSoundIcon(festival_info, message); break;
+	    case MSGTYPE_CHAR: r = festivalCharacter(festival_info, message); break;
+	    case MSGTYPE_KEY: r = festivalKey(festival_info, message); break;
+	    case MSGTYPE_SPELL: r = festivalSpell(festival_info, message); break;
+	    }
+	if (r < 0){
+	    DBG("Couldn't process the request to say the object.");
+	    CLEAN_UP(-1);
+	}
+    }
+    
 
-    pos = 0;
     while(1){
-        DBG("  Looping...\n");
-        o_bytes = bytes;
 
-        if (msgtype == MSGTYPE_TEXT)
-            bytes = module_get_message_part(message, buf, &pos, FestivalMaxChunkLength, dividers);
-        else{
-            bytes = module_get_message_part(message, buf, &pos, FestivalMaxChunkLength, "");
-        }
-        if (current_index_mark != -1) pause = current_index_mark;
-        if (bytes == 0){
-            if (msgtype == MSGTYPE_TEXT)
-                bytes = module_get_message_part(message, buf, &pos, FestivalMaxChunkLength, dividers);
-            else
-                return -1;
-        }
-        
-        DBG("returned %d bytes\n", bytes); 
+	DBG("Retrieving data\n");
 
-        assert( ! ((bytes > 0) && (buf == NULL)) );
+	/* (speechd-next) */
+	if (is_text(msgtype)){
+	    DBG("Getting data in multi mode");
+	    fwave = festivalGetDataMulti(festival_info, &callback, &festival_stop, FestivalReopenSocket);
 
-        if (first_run){
-            DBG("Synthesizing: |%s|", buf);
-            if (bytes > 0){
-                switch(msgtype)
-                    {
-                    case MSGTYPE_CHAR:
-                    case MSGTYPE_KEY:
-                    case MSGTYPE_SOUND_ICON:
-                        fwave = cache_lookup(buf, msgtype, 1);
-                        if (fwave != NULL){
-                            if (fwave->num_samples != 0){
-                                ret = module_parent_send_samples(dpipe, fwave->samples,
-                                                                 fwave->num_samples, fwave->sample_rate,
-								 festival_volume);
-                                if (ret == -1) terminate = 1;
-                                if(FestivalDebugSaveOutput){
-                                    char filename_debug[256];
-                                    sprintf(filename_debug, "/tmp/debug-festival-%d.snd", debug_count++);
-                                    save_FT_Wave_snd(fwave, filename_debug);
-                                }
-               
-                                DBG("parent (1): Sent %d bytes\n", ret);            
-                                ret = module_parent_dp_write(dpipe, "\r\nOK_SPEECHD_DATA_SENT\r\n", 24);
-                                if (ret == -1) terminate = 1;
+	    if (callback != NULL){
+		DBG("Callback detected: %s", callback);
+		l = strlen(callback);
+		if (l<4) continue;	/* some other index mark */
+		if (strncmp(callback, "sdm_", 3)) continue;	/* some other index mark */
+		h = strtol(callback+3, &tptr, 10);
+		if (tptr != callback+3) current_index_mark = h;
+		DBG("Current index mark is set to %d", current_index_mark);
+		if(*pause_requested){
+		    DBG("Pause requested, not sending more data to child");
+		    CLEAN_UP(0);
+		}else{
+		    continue;
+		}
+	    }
+	}else{			/* is event */
+	    DBG("Getting data in single mode");
+	    fwave = festivalStringToWaveGetData(festival_info);
+	    terminate = 1;
+	    callback = NULL;
+	}
 
-                                if (terminate != 1) terminate = module_parent_wait_continue(dpipe);
-                                DBG("End of data in parent (1), closing pipes [%d:%d]", terminate, bytes);
-                                module_parent_dp_close(dpipe);
-                            }
-                            return 0;
-                        }
-                    }
+	if (fwave == NULL){
+	    DBG("End of sound samples, returning...");
+	    CLEAN_UP(0);
+	}
+	
+	if (msgtype == MSGTYPE_CHAR || msgtype == MSGTYPE_KEY ||
+	    msgtype == MSGTYPE_SOUND_ICON){
+	    DBG("Storing record for %s in cache\n", message);
+	    wave_cached = 1;
+	    cache_insert(strdup(message), msgtype, fwave);
+	}
+	
+	DBG("Sending message to child in wav: %d bytes\n",
+	    (fwave->num_samples) * sizeof(short));
+	
+	if (fwave->num_samples != 0){
+	    
+	    if(FestivalDebugSaveOutput){
+		char filename_debug[256];
+		sprintf(filename_debug, "/tmp/debug-festival-%d.snd", debug_count++);
+		save_FT_Wave_snd(fwave, filename_debug);
+	    }
+	    
+	    ret = module_parent_send_samples(dpipe, fwave->samples,
+					     fwave->num_samples, fwave->sample_rate,
+					     festival_volume);
+	    if (ret == -1) CLEAN_UP(-1);
+	    DBG("parent: Sent %d bytes\n", ret);            
+	    
+	    ret = module_parent_dp_write(dpipe, "\r\nOK_SPEECHD_DATA_SENT\r\n", 24);
+	    if (ret == -1) CLEAN_UP(-1);
+	    ret = module_parent_wait_continue(dpipe);
+	    if (ret != 0) CLEAN_UP(-1);
+	}            
 
-                switch(msgtype)
-                    {
-                    case MSGTYPE_TEXT: r = festivalStringToWaveRequest(festival_info, buf); break;
-                    case MSGTYPE_SOUND_ICON: r = festivalSoundIcon(festival_info, buf); break;
-                    case MSGTYPE_CHAR: r = festivalCharacter(festival_info, buf); break;
-                    case MSGTYPE_KEY: r = festivalKey(festival_info, buf); break;
-                    case MSGTYPE_SPELL: r = festivalSpell(festival_info, buf); break;
-                    }
-            }
-            DBG("s-returned %d\n", r);
-            first_run = 0;
-        }else{                                       
-            DBG("Retrieving data\n");
-            if (o_bytes > 0) fwave = festivalStringToWaveGetData(festival_info, &cont, 0);
-            else fwave = NULL;
-
-            DBG("ss-returned %d\n", r);
-            DBG("Ok, next step...\n");
-
-            /* This is not synchronous here, but there is no other way to do it */
-            DBG("Sending request for synthesis");
-            if (bytes > 0){
-                r = festivalStringToWaveRequest(festival_info, buf);
-            }                
-
-            /* fwave can be NULL if the given text didn't produce any sound
-               output, e.g. this text: "." */
-            if (fwave != NULL){
-                DBG("Sending buf to child in wav: %d bytes\n",
-                    (fwave->num_samples) * sizeof(short));
-                if (fwave->num_samples != 0){
-                    ret = module_parent_send_samples(dpipe, fwave->samples,
-                                                     fwave->num_samples, fwave->sample_rate,
-						     festival_volume);
-                    if (ret == -1) terminate = 1;
-                    if(FestivalDebugSaveOutput){
-                        char filename_debug[256];
-                        sprintf(filename_debug, "/tmp/debug-festival-%d.snd", debug_count++);
-                        save_FT_Wave_snd(fwave, filename_debug);
-                    }
-               
-                    DBG("parent: Sent %d bytes\n", ret);            
-                    ret = module_parent_dp_write(dpipe, "\r\nOK_SPEECHD_DATA_SENT\r\n", 24);
-                    if (ret == -1) terminate = 1;
-
-                    if (terminate != 1) terminate = module_parent_wait_continue(dpipe);
-                }
-
-                if (msgtype == MSGTYPE_CHAR || msgtype == MSGTYPE_KEY ||
-                    msgtype == MSGTYPE_SOUND_ICON){
-                    DBG("Storing record for %s in cache\n", buf);
-                    cache_insert(strdup(buf), msgtype, fwave);
-                }else{
-                    delete_FT_Wave(fwave);
-                }
-            }            
-        }
-        if ((pause >= 0) && (*pause_requested)){
-            if (bytes > 0){
-                fwave = festivalStringToWaveGetData(festival_info, &cont, 0);    
-                delete_FT_Wave(fwave);
-            }
-            module_parent_dp_close(dpipe);
-            *pause_requested = 0;
-            current_index_mark = pause;        
-            DBG("Pause requested in parent, index mark %d\n", current_index_mark);
-            return current_index_mark;
-        }
-
-        if (terminate && (bytes > 0)){
-            if (o_bytes > 0){
-                fwave = festivalStringToWaveGetData(festival_info, &cont, 0);    
-                delete_FT_Wave(fwave);
-            }
-        }
-
-        if (terminate || (bytes == -1)){
-            DBG("End of data in parent, closing pipes [%d:%d]", terminate, bytes);
-            module_parent_dp_close(dpipe);
-            break;
-        }            
-    }    
+	if (terminate){
+	    DBG("Ok, end of samples, returning");
+	    CLEAN_UP(0);
+	}
+    }
 }
 
 void
 festival_set_language(char* language)
 {   
-    FestivalSetLanguage(festival_info, language, &festival_coding);
+    FestivalSetLanguage(festival_info, language, NULL);
 }
 
 void
@@ -503,7 +595,7 @@ festival_set_voice(EVoiceType voice)
     char* voice_name;
 
     voice_name = EVoice2str(voice);
-    FestivalSetVoice(festival_info, voice_name, &festival_coding);
+    FestivalSetVoice(festival_info, voice_name, NULL);
     xfree(voice_name);
 
     /* Because the new voice can use diferent bitrate */
@@ -570,6 +662,8 @@ festival_conv(FT_Wave *fwave)
     return ret;
 }
 
+
+
 /* --- Cache related functions --- */
 
 void
@@ -655,7 +749,7 @@ cache_debug_foreach_list_score(gpointer a, gpointer user)
 
     t = time(NULL);
     DBG("key: %s      -> score %f (count: %d, dtime: %d)", A->key, ((float) A->count  / (float)(t - A->start)),
-        A->count, (t - A->start));
+        (int) A->count, (int) (t - A->start));
 }
 
 /* Remove 1/3 of the least used (according to cache_counter_comp) entries 
@@ -714,8 +808,12 @@ cache_gen_key(EMessageType type)
     if (FestivalCacheDistinguishRate) krate = msg_settings.rate;
 
     if (type == MSGTYPE_CHAR) ktype = 'c';
-    if (type == MSGTYPE_KEY) ktype = 'k';
-    if (type == MSGTYPE_SOUND_ICON) ktype = 's';
+    else if (type == MSGTYPE_KEY) ktype = 'k';
+    else if (type == MSGTYPE_SOUND_ICON) ktype = 's';
+    else{
+	DBG("Invalid message type for cache_gen_key()");
+	return NULL;
+    }
 
     key = g_strdup_printf("%c_%s_%d_%d_%d", ktype, msg_settings.language, kvoice,
                                 krate, kpitch);
@@ -735,12 +833,11 @@ cache_insert(char* key, EMessageType msgtype, FT_Wave *fwave)
     if (FestivalCacheOn == 0) return 0;
 
     if (key == NULL) return -1;
+    if (fwave == NULL) return -1;
 
     /* Check if the entry isn't present already */
-    if (cache_lookup(key, msgtype, 0) != NULL){
-        delete_FT_Wave(fwave);
+    if (cache_lookup(key, msgtype, 0) != NULL)
         return 0;
-    }
 
     key_table = cache_gen_key(msgtype);
 
@@ -750,6 +847,7 @@ cache_insert(char* key, EMessageType msgtype, FT_Wave *fwave)
     if ((FestivalCache.size + fwave->num_samples * sizeof(short))
         > (FestivalCacheMaxKBytes*1024))
         if (cache_clean(fwave->num_samples * sizeof(short)) != 0) return -1;
+    
 
     /* Select the right table according to language, voice, etc. or create a new one*/
     cache = g_hash_table_lookup(FestivalCache.caches, key_table);
@@ -760,7 +858,7 @@ cache_insert(char* key, EMessageType msgtype, FT_Wave *fwave)
         xfree(key_table);
     }
 
-    /* Fill the Counter Entry structure that will later allow us to remove
+    /* Fill the CounterEntry structure that will later allow us to remove
        the less used entries from cache */
     centry = (TCounterEntry*) xmalloc(sizeof(TCounterEntry));
     centry->start = time(NULL);
@@ -782,7 +880,7 @@ cache_insert(char* key, EMessageType msgtype, FT_Wave *fwave)
 
 /* Retrieve wave from the cache */
 FT_Wave*
-cache_lookup(char *key, EMessageType msgtype, int add_counter)
+cache_lookup(const char *key, EMessageType msgtype, int add_counter)
 {
     GHashTable *cache;
     TCacheEntry *entry;
@@ -809,4 +907,84 @@ cache_lookup(char *key, EMessageType msgtype, int add_counter)
     return entry->fwave;
 }
 
+
+int
+init_festival_standalone()
+{
+    int ret;
+    int fr;
+
+    if ( (pipe(module_p.pipe_in) != 0) 
+          || ( pipe(module_p.pipe_out) != 0)  ){
+        DBG("Can't open pipe! Module not loaded.");
+        return -1;
+    }     
+
+    DBG("Starting Festival as a child process");
+
+    fr = fork();
+    switch(fr){
+    case -1: DBG("ERROR: Can't fork! Module not loaded."); return -1;
+    case 0:
+        ret = dup2(module_p.pipe_in[0], 0);
+        close(module_p.pipe_in[0]);
+        close(module_p.pipe_in[1]);
+ 
+        ret = dup2(module_p.pipe_out[1], 1);
+        close(module_p.pipe_out[1]);
+        close(module_p.pipe_out[0]);        
+        
+	/* TODO: fix festival hardcoded path */
+	if (execlp("festival", "", (char *) 0) == -1)
+	    exit(1);	
+                
+    default:
+        festival_process_pid = fr;
+        close(module_p.pipe_in[0]);
+        close(module_p.pipe_out[1]);
+
+        usleep(100);            /* So that the other child has at least time to fail
+                                   with the execlp */
+        ret = waitpid(module_p.pid, NULL, WNOHANG);
+        if (ret != 0){
+            DBG("Can't execute festival. Bad filename in configuration?");            
+            return -1;
+        }
+	
+        return 0;
+    }
+    
+    assert(0);
+}
+
+int
+init_festival_socket()
+{
+    int r;
+
+    /* Init festival and register a new voice */
+    festival_info = festivalDefaultInfo();
+    festival_info->server_host = FestivalServerHost;
+    festival_info->server_port = FestivalServerPort;
+    
+    festival_info = festivalOpen(festival_info);
+    if (festival_info == NULL) return -1;
+    r = FestivalSetMultiMode(festival_info, "t");
+    if (r != 0) return -2;
+    
+    DBG("FestivalServerHost = %s\n", FestivalServerHost);
+    DBG("FestivalServerPort = %d\n", FestivalServerPort);
+
+    return 0;
+}
+
+int
+stop_festival_local()
+{
+    if (festival_process_pid > 0)
+	kill(festival_process_pid, SIGINT);
+    return 0;
+}
+
+
 #include "module_main.c"
