@@ -18,11 +18,14 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: module_main.c,v 1.3 2004/04/04 21:11:49 hanke Exp $
+ * $Id: module_main.c,v 1.5 2004/07/21 08:14:41 hanke Exp $
  */
 
+/* So that gcc doesn't comply */
+int getline(char**, int*, FILE*);
+
 #define PROCESS_CMD(command, function) if (!strcmp(cmd_buf, #command"\n")){ \
- if (printf("%s\n", function()) < 0){ \
+ if (printf("%s\n", (char*) function()) < 0){ \
      DBG("Broken pipe, exiting...\n"); \
      module_close(2); \
  } \
@@ -34,10 +37,13 @@ main(int argc, char *argv[])
 {
     char *cmd_buf;
     int ret;
+    int ret_init;
     int n;
-    configfile_t *configfile = NULL;
     char *configfilename;
+    char *status_info;
 
+    module_num_dc_options = 0;
+    
     if (argc >= 2){
         char *tailptr;
         SPDSemaphore = strtol(argv[1], &tailptr, 0);
@@ -55,7 +61,6 @@ main(int argc, char *argv[])
     ret = module_load();
     if (ret == -1) module_close(1);
 
-
     if (configfilename != NULL){
         /* Add the LAST option */
         module_dc_options = add_config_option(module_dc_options,
@@ -67,10 +72,10 @@ main(int argc, char *argv[])
                 DBG("Error reading config file\n");
                 module_close(1);
             }
-        dotconf_cleanup(configfile);
-        DBG("Configuration (pre) has been read from \"%s\"\n", configfilename);    
-        
-        xfree(configfilename);
+	    dotconf_cleanup(configfile);
+	    DBG("Configuration (pre) has been read from \"%s\"\n", configfilename);    
+	    
+	    xfree(configfilename);
         }else{
             DBG("Can't read specified config file!\n");        
         }
@@ -78,8 +83,35 @@ main(int argc, char *argv[])
         DBG("No config file specified, using defaults...\n");        
     }
     
-    ret = module_init();
-    if (ret == -1) module_close(1);
+    ret_init = module_init(&status_info);
+
+    cmd_buf = NULL;  n=0;
+    ret = getline(&cmd_buf, &n, stdin);
+    if (ret == -1){
+	DBG("Broken pipe when reading INIT, exiting... \n");
+	module_close(2); 
+    }
+
+    if (!strcmp(cmd_buf, "INIT\n")){
+	if (ret_init == 0){
+	    printf("299-%s\n", status_info);
+	    ret = printf("%s\n", "299 OK LOADED SUCCESSFULLY\n");
+	}else{
+	    printf("399-%s\n", status_info);
+	    ret = printf("%s\n", "399 ERR CANT INIT MODULE\n");
+	    return -1;
+	}
+      	xfree(status_info);
+
+	if (ret < 0){ 
+	    DBG("Broken pipe, exiting...\n");
+            module_close(2); 
+	}
+	fflush(stdout);
+    }else{
+	DBG("ERROR: Wrong communication from module client: didn't call INIT\n");
+	module_close(3);
+    }
 
     while(1){
         cmd_buf = NULL;  n=0;
@@ -87,9 +119,9 @@ main(int argc, char *argv[])
         if (ret == -1){
             DBG("Broken pipe, exiting... \n");
             module_close(2); 
-        } 
+        }
 
-        PROCESS_CMD(SPEAK, do_speak) 
+	PROCESS_CMD(SPEAK, do_speak) 
         else PROCESS_CMD(SOUND_ICON, do_sound_icon)
         else PROCESS_CMD(CHAR, do_char)
         else PROCESS_CMD(KEY, do_key)
