@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: parse.c,v 1.61 2004/04/17 14:54:03 hanke Exp $
+ * $Id: parse.c,v 1.63 2004/10/12 20:27:36 hanke Exp $
  */
 
 #include "speechd.h"
@@ -138,10 +138,10 @@ parse(const char *buf, const int bytes, const int fd)
         if(((bytes >= 5)&&((!strncmp(buf, "\r\n.\r\n", bytes))))||(end_data == 1)
            ||((bytes == 3)&&(!strncmp(buf, ".\r\n", bytes)))){
 
-            MSG(4,"Finishing data");
+            MSG(5,"Finishing data");
             end_data = 0;
             /* Set the flag to command mode */
-            MSG(4, "Switching back to command mode...");
+            MSG(5, "Switching back to command mode...");
             SpeechdSocket[fd].awaiting_data = 0;
 
             /* Prepare element (text+settings commands) to be queued. */
@@ -164,7 +164,7 @@ parse(const char *buf, const int bytes, const int fd)
 	    /* TODO: Unify this with the above copying */
 	    new->buf = deescape_dot(new->buf);
 
-            MSG(4, "New buf is now: |%s|", new->buf);		
+            MSG(5, "New buf is now: |%s|", new->buf);		
             if(queue_message(new, fd, 1, MSGTYPE_TEXT, reparted) != 0){
                 if(SPEECHD_DEBUG) FATAL("Can't queue message\n");
                 free(new->buf);
@@ -183,7 +183,7 @@ parse(const char *buf, const int bytes, const int fd)
                 if(pos = strstr(buf,"\r\n.\r\n")){	
                     real_bytes=pos-buf;
                     end_data=1;		
-                    MSG(4,"Command in data caught");
+                    MSG(5,"Command in data caught");
                 }else{
                     real_bytes = bytes;
                 }
@@ -532,6 +532,24 @@ parse_set(const char *buf, const int bytes, const int fd)
         if (ret) return ERR_COULDNT_SET_PAUSE_CONTEXT;
         return OK_PAUSE_CONTEXT_SET;
     }
+    else if (!strcmp(set_sub, "ssml_mode")){
+	char *ssml_mode_s;
+	int ssml_mode;
+
+        GET_PARAM_STR(ssml_mode_s, 3, CONV_DOWN);
+
+        if(TEST_CMD(ssml_mode_s, "on")) ssml_mode = 1;
+        else if(TEST_CMD(ssml_mode_s, "off")) ssml_mode = 0;        
+        else{
+            spd_free(ssml_mode_s);
+            return ERR_PARAMETER_NOT_ON_OFF;
+        }
+
+        SSIP_SET_COMMAND(ssml_mode);
+
+        if (ret) return ERR_COULDNT_SET_SSML_MODE;
+        return OK_SSML_MODE_SET;
+    }
     else{
         return ERR_PARAMETER_INVALID;
     }
@@ -546,7 +564,7 @@ parse_stop(const char *buf, const int bytes, const int fd)
     int uid = 0;
     char *who_s;
 
-    MSG(4, "Stop received from fd %d.", fd);
+    MSG(5, "Stop received from fd %d.", fd);
 
     GET_PARAM_STR(who_s, 1, CONV_DOWN);
 
@@ -697,7 +715,7 @@ parse_general_event(const char *buf, const int bytes, const int fd, EMessageType
 
     if(queue_message(msg, fd, 1, type, SpeechdSocket[fd].inside_block)){
         if (SPEECHD_DEBUG) FATAL("Couldn't queue message\n");
-        MSG(2, "Couldn't queue message!\n");            
+        MSG(2, "Error: Couldn't queue message!\n");            
     }   
 
     return OK_MESSAGE_QUEUED;
@@ -810,7 +828,7 @@ deescape_dot(char *otext)
 
     if (otext == NULL) return NULL;
 
-    MSG2(5, "escaping", "Incomming text: |%s|", otext);
+    MSG2(6, "escaping", "Incomming text: |%s|", otext);
 
     ootext = otext;
 
@@ -830,26 +848,26 @@ deescape_dot(char *otext)
         }
     }
 
-    MSG2(5, "escaping", "Altering text (I): |%s|", ntext->str);
+    MSG2(6, "escaping", "Altering text (I): |%s|", ntext->str);
 
     while (seq = strstr(otext, "\r\n..\r\n")){
         *seq = 0;
         g_string_append(ntext, otext);
         g_string_append(ntext, "\r\n.\r\n");
 
-        MSG2(5, "escaping", "Altering text (II) / 1: |%s|", otext);    
+        MSG2(6, "escaping", "Altering text (II) / 1: |%s|", otext);    
         otext = seq + 6;
-        MSG2(5, "escaping", "Altering text (II) / 2: |%s|", otext);    
+        MSG2(6, "escaping", "Altering text (II) / 2: |%s|", otext);    
     }
 
-    MSG2(5, "escaping", "Altering text (II): |%s|", ntext->str);    
+    MSG2(6, "escaping", "Altering text (II): |%s|", ntext->str);    
 
     len = strlen(otext);
     if (len >= 4){
         if ((otext[len-4] == '\r') && (otext[len-3] == '\n')
             && (otext[len-2] == '.') && (otext[len-1] == '.')){
             otext[len-1] = 0;
-            MSG2(5, "escaping", "Altering text (II-b) otext: |%s|", otext);    
+            MSG2(6, "escaping", "Altering text (II-b) otext: |%s|", otext);    
         }
     }
 
@@ -862,7 +880,7 @@ deescape_dot(char *otext)
     }
     g_string_free(ntext, 0);
 
-    MSG2(5, "escaping", "Altered text: |%s|", ret);
+    MSG2(6, "escaping", "Altered text: |%s|", ret);
 
     return ret;
 }
